@@ -1,4 +1,4 @@
-"""This module contains utilities for extracting information from AST nodes."""
+"""This module contains utilities for extracting information from nodes."""
 
 from __future__ import annotations
 
@@ -63,6 +63,7 @@ from ast import Tuple as NodeTuple
 from ast import UAdd as NodeUAdd
 from ast import UnaryOp as NodeUnaryOp
 from ast import USub as NodeUSub
+from ast import Yield as NodeYield
 from ast import arguments as NodeArguments
 from ast import comprehension as NodeComprehension
 from ast import keyword as NodeKeyword
@@ -248,8 +249,10 @@ class ObjectKind(enum.Enum):
     CLASSMETHOD: str = "classmethod"
     METHOD_DESCRIPTOR: str = "method_descriptor"
     METHOD: str = "method"
+    BUILTIN_METHOD: str = "builtin_method"
     COROUTINE: str = "coroutine"
     FUNCTION: str = "function"
+    BUILTIN_FUNCTION: str = "builtin_function"
     CACHED_PROPERTY: str = "cached_property"
     PROPERTY: str = "property"
     ATTRIBUTE: str = "attribute"
@@ -272,6 +275,9 @@ class ObjectNode:
         parent: The parent node.
     """
 
+    # low level stuff known to cause issues when resolving aliases
+    exclude_specials = {"__builtins__", "__loader__", "__spec__"}
+
     def __init__(self, obj: Any, name: str, parent: ObjectNode | None = None) -> None:
         """
         Initialize the object.
@@ -316,10 +322,14 @@ class ObjectNode:
             return ObjectKind.METHOD_DESCRIPTOR
         if self.is_method:
             return ObjectKind.METHOD
+        if self.is_builtin_method:
+            return ObjectKind.BUILTIN_METHOD
         if self.is_coroutine:
             return ObjectKind.COROUTINE
         if self.is_function:
             return ObjectKind.FUNCTION
+        if self.is_builtin_function:
+            return ObjectKind.BUILTIN_FUNCTION
         if self.is_cached_property:
             return ObjectKind.CACHED_PROPERTY
         if self.is_property:
@@ -335,7 +345,7 @@ class ObjectNode:
         """
         children = []
         for name, member in inspect.getmembers(self.obj):
-            if self._pick_member(member):
+            if self._pick_member(name, member):
                 children.append(ObjectNode(member, name, parent=self))
         return children
 
@@ -369,6 +379,16 @@ class ObjectNode:
         """
         return inspect.isfunction(self.obj)
 
+    @cached_property
+    def is_builtin_function(self) -> bool:
+        """
+        Tell if this node's object is a builtin function.
+
+        Returns:
+            If this node's object is a builtin function.
+        """
+        return inspect.isbuiltin(self.obj)
+
     @cached_property
     def is_coroutine(self) -> bool:
         """
@@ -433,6 +453,16 @@ class ObjectNode:
         """
         return inspect.ismethoddescriptor(self.obj)
 
+    @cached_property
+    def is_builtin_method(self) -> bool:
+        """
+        Tell if this node's object is a builtin method.
+
+        Returns:
+            If this node's object is a builtin method.
+        """
+        return self.is_builtin_function and self.parent_is_class
+
     @cached_property
     def is_staticmethod(self) -> bool:
         """
@@ -441,7 +471,7 @@ class ObjectNode:
         Returns:
             If this node's object is a staticmethod.
         """
-        if not self.parent:
+        if self.parent is None:
             return False
         try:
             self_from_parent = self.parent.obj.__dict__.get(self.name, None)  # noqa: WPS609
@@ -457,7 +487,7 @@ class ObjectNode:
         Returns:
             If this node's object is a classmethod.
         """
-        if not self.parent:
+        if self.parent is None:
             return False
         try:
             self_from_parent = self.parent.obj.__dict__.get(self.name, None)  # noqa: WPS609
@@ -468,11 +498,16 @@ class ObjectNode:
     @cached_property
     def _ids(self) -> set[int]:
         if self.parent is None:
-            return {id(self)}
-        return {id(self)} | self.parent._ids  # noqa: WPS437
+            return {id(self.obj)}
+        return {id(self.obj)} | self.parent._ids  # noqa: WPS437
 
-    def _pick_member(self, member: Any) -> bool:
-        return member is not type and member is not object and id(member) not in self._ids
+    def _pick_member(self, name: str, member: Any) -> bool:
+        return (
+            name not in self.exclude_specials
+            and member is not type
+            and member is not object
+            and id(member) not in self._ids
+        )
 
 
 def _join(sequence, item):
@@ -484,6 +519,33 @@ def _join(sequence, item):
     return new_sequence
 
 
+if sys.version_info < (3, 8):
+
+    def parse__all__(node: NodeAssign) -> set[str]:  # noqa: WPS116,WPS120
+        """Get the values declared in `__all__`.
+
+        Parameters:
+            node: The assignment node.
+
+        Returns:
+            A set of names.
+        """
+        return {elt.s for elt in node.value.elts}  # type: ignore[attr-defined]
+
+else:
+
+    def parse__all__(node: NodeAssign) -> set[str]:  # noqa: WPS116,WPS120,WPS440
+        """Get the values declared in `__all__`.
+
+        Parameters:
+            node: The assignment node.
+
+        Returns:
+            A set of names.
+        """
+        return {elt.value for elt in node.value.elts}  # type: ignore[attr-defined]
+
+
 # ==========================================================
 # annotations
 def _get_attribute_annotation(node: NodeAttribute, parent: Module | Class) -> Expression:
@@ -722,6 +784,10 @@ def _get_constant_value(node: NodeConstant) -> str:
     return repr(node.value)
 
 
+def _get_constant_value_no_repr(node: NodeConstant) -> str:
+    return node.value
+
+
 def _get_dict_value(node: NodeDict) -> str:
     pairs = zip(node.keys, node.values)
     gen = (f"{'None' if key is None else get_value(key)}: {get_value(value)}" for key, value in pairs)  # noqa: WPS509
@@ -790,7 +856,10 @@ def _get_isnot_value(node: NodeIsNot) -> str:
 
 
 def _get_joinedstr_value(node: NodeJoinedStr) -> str:
-    return "".join(get_value(value) for value in node.values)
+    _node_value_map[NodeConstant] = _get_constant_value_no_repr
+    result = repr("".join(get_value(value) for value in node.values))
+    _node_value_map[NodeConstant] = _get_constant_value
+    return result
 
 
 def _get_keyword_value(node: NodeKeyword) -> str:
@@ -885,7 +954,10 @@ def _get_sub_value(node: NodeSub) -> str:
 
 
 def _get_subscript_value(node: NodeSubscript) -> str:
-    return f"{get_value(node.value)}[{get_value(node.slice).strip('()')}]"
+    subscript = get_value(node.slice)
+    if isinstance(subscript, str):
+        subscript = subscript.strip("()")
+    return f"{get_value(node.value)}[{subscript}]"
 
 
 def _get_tuple_value(node: NodeTuple) -> str:
@@ -904,6 +976,12 @@ def _get_usub_value(node: NodeUSub) -> str:
     return "-"
 
 
+def _get_yield_value(node: NodeYield) -> str:
+    if node.value is None:
+        return repr(None)
+    return get_value(node.value)
+
+
 _node_value_map: dict[Type, Callable[[Any], str]] = {
     type(None): lambda _: repr(None),
     NodeAdd: _get_add_value,
@@ -961,6 +1039,7 @@ _node_value_map: dict[Type, Callable[[Any], str]] = {
     NodeUAdd: _get_uadd_value,
     NodeUnaryOp: _get_unaryop_value,
     NodeUSub: _get_usub_value,
+    NodeYield: _get_yield_value,
 }
 
 # TODO: remove once Python 3.8 support is dropped
