@@ -25,6 +25,7 @@ from griffe.agents.nodes import (
     get_names,
     get_parameter_default,
     get_value,
+    parse__all__,
 )
 from griffe.collections import LinesCollection, ModulesCollection
 from griffe.dataclasses import (
@@ -176,7 +177,8 @@ class Visitor(BaseVisitor):  # noqa: WPS338
         """
         for before_visitor in self.extensions.before_children_visit:
             before_visitor.visit(node)
-        super().generic_visit(node)
+        for child in node.children:  # type: ignore[attr-defined]  # noqa: WPS437
+            self.visit(child)
         for after_visitor in self.extensions.after_children_visit:
             after_visitor.visit(node)
 
@@ -379,7 +381,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
             )
         self.generic_visit(node)
 
-    def visit_importfrom(self, node: ast.ImportFrom) -> None:
+    def visit_importfrom(self, node: ast.ImportFrom) -> None:  # noqa: WPS231
         """Visit an "import from" node.
 
         Parameters:
@@ -387,11 +389,30 @@ class Visitor(BaseVisitor):  # noqa: WPS338
         """
         for name in node.names:
             alias_name = name.asname or name.name
-            alias_path = f"{node.module}.{name.name}"
-            self.current.imports[name.asname or name.name] = alias_path
+            level = node.level
+            module_path = node.module
+            if level > 0:
+                if module_path is None:
+                    level -= 1
+                parent: Module = self.current.module
+                if parent.is_package or parent.is_subpackage:
+                    level -= 1
+                while level > 0:
+                    parent = parent.parent  # type: ignore[assignment]
+                    level -= 1
+                if module_path:
+                    module_path = f"{parent.path}.{module_path}"
+                else:
+                    module_path = parent.path
+            if alias_name == "*":
+                alias_name = module_path.replace(".", "/") + "/*"  # type: ignore[union-attr]
+                alias_path = module_path
+            else:
+                alias_path = f"{module_path}.{name.name}"
+                self.current.imports[alias_name] = alias_path
             self.current[alias_name] = Alias(
                 alias_name,
-                alias_path,
+                alias_path,  # type: ignore[arg-type]
                 lineno=node.lineno,
                 endlineno=node.end_lineno,  # type: ignore[attr-defined]
             )
@@ -468,7 +489,7 @@ class Visitor(BaseVisitor):  # noqa: WPS338
 
             if name == "__all__":
                 with suppress(AttributeError):
-                    parent.exports = {elt.value for elt in node.value.elts}  # type: ignore[union-attr]
+                    parent.exports = parse__all__(node)  # type: ignore[arg-type]
 
     def visit_assign(self, node: ast.Assign) -> None:
         """Visit an assignment node.
