@@ -14,9 +14,9 @@
 from __future__ import annotations
 
 import argparse
-import asyncio
 import json
 import logging
+import os
 import sys
 from pathlib import Path
 from typing import Any, Sequence
@@ -25,13 +25,14 @@ from griffe.agents.extensions import Extensions
 from griffe.agents.extensions.base import load_extensions
 from griffe.docstrings.parsers import Parser
 from griffe.encoders import Encoder
-from griffe.loader import AsyncGriffeLoader, GriffeLoader
+from griffe.exceptions import ExtensionError
+from griffe.loader import GriffeLoader
 from griffe.logger import get_logger
 
 logger = get_logger(__name__)
 
 
-def _print_data(data, output_file):
+def _print_data(data: str, output_file: str):
     if output_file is sys.stdout:
         print(data)
     else:
@@ -39,52 +40,39 @@ def _print_data(data, output_file):
             print(data, file=fd)
 
 
-async def _load_packages_async(
-    packages: Sequence[str],
-    extensions: Extensions | None,
-    search_paths: Sequence[str],
-    docstring_parser: Parser | None,
-    docstring_options: dict[str, Any],
-):
-    loader = AsyncGriffeLoader(
-        extensions=extensions,
-        docstring_parser=docstring_parser,
-        docstring_options=docstring_options,
-    )
-    loaded = {}
-    for package in packages:
-        logger.info(f"Loading package {package}")
-        try:
-            module = await loader.load_module(package, search_paths=search_paths)
-        except ModuleNotFoundError:
-            logger.error(f"Could not find package {package}")
-        else:
-            loaded[module.name] = module
-    return loaded
-
-
 def _load_packages(
     packages: Sequence[str],
     extensions: Extensions | None,
     search_paths: Sequence[str],
     docstring_parser: Parser | None,
     docstring_options: dict[str, Any],
+    resolve_aliases: bool = True,
+    only_exported: bool = True,
+    only_known_modules: bool = True,
 ):
     loader = GriffeLoader(
         extensions=extensions,
         docstring_parser=docstring_parser,
         docstring_options=docstring_options,
     )
-    loaded = {}
     for package in packages:
         logger.info(f"Loading package {package}")
         try:
-            module = loader.load_module(package, search_paths=search_paths)
-        except ModuleNotFoundError:
-            logger.error(f"Could not find package {package}")
+            loader.load_module(package, search_paths=search_paths)
+        except ModuleNotFoundError as error:
+            logger.error(f"Could not find package {package}: {error}")
+        except ImportError as error:
+            logger.error(f"Tried but could not import package {package}: {error}")
+    if resolve_aliases:
+        unresolved, iterations = loader.resolve_aliases(only_exported, only_known_modules)
+        if unresolved:
+            logger.info(f"{len(unresolved)} aliases were still unresolved after {iterations} iterations")
         else:
-            loaded[module.name] = module
-    return loaded
+            logger.info(f"All aliases were resolved after {iterations} iterations")
+    return loader.modules_collection.members
+
+
+_level_choices = ("DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL")
 
 
 def get_parser() -> argparse.ArgumentParser:
@@ -108,6 +96,20 @@ def get_parser() -> argparse.ArgumentParser:
         action="store_true",
         help="Whether to append sys.path to search paths specified with -s.",
     )
+    parser.add_argument(
+        "-d",
+        "--docstyle",
+        default=None,
+        type=Parser,
+        help="The docstring style to parse.",
+    )
+    parser.add_argument(
+        "-D",
+        "--docopts",
+        default={},
+        type=json.loads,
+        help="The options for the docstring parser.",
+    )
     parser.add_argument(
         "-e",
         "--extensions",
@@ -128,26 +130,21 @@ def get_parser() -> argparse.ArgumentParser:
         action="help",
         help="Show this help message and exit.",
     )
+    parser.add_argument(
+        "-L",
+        "--log-level",
+        metavar="LEVEL",
+        default=os.getenv("GRIFFE_LOG_LEVEL", "INFO").upper(),
+        choices=_level_choices,
+        type=str.upper,
+        help="Set the log level: DEBUG, INFO, WARNING, ERROR, CRITICAL.",
+    )
     parser.add_argument(
         "-o",
         "--output",
         default=sys.stdout,
         help="Output file. Supports templating to output each package in its own file, with {{package}}.",
     )
-    parser.add_argument(
-        "-d",
-        "--docstyle",
-        default=None,
-        type=Parser,
-        help="The docstring style to parse.",
-    )
-    parser.add_argument(
-        "-D",
-        "--docopts",
-        default={},
-        type=json.loads,
-        help="The options for the docstring parser.",
-    )
     parser.add_argument(
         "-s",
         "--search",
@@ -174,7 +171,17 @@ def main(args: list[str] | None = None) -> int:  # noqa: WPS231
     parser = get_parser()
     opts: argparse.Namespace = parser.parse_args(args)
 
-    logging.basicConfig(format="%(levelname)-10s %(message)s", level=logging.INFO)  # noqa: WPS323
+    try:
+        level = getattr(logging, opts.log_level)
+    except AttributeError:
+        choices = "', '".join(_level_choices)
+        print(
+            f"griffe: error: env var GRIFFE_LOG_LEVEL: invalid level '{opts.log_level}' (choose from '{choices}')",
+            file=sys.stderr,
+        )
+        sys.exit(1)
+
+    logging.basicConfig(format="%(levelname)-10s %(message)s", level=level)  # noqa: WPS323
 
     output = opts.output
 
@@ -186,14 +193,13 @@ def main(args: list[str] | None = None) -> int:  # noqa: WPS231
     if opts.append_sys_path:
         search.extend(sys.path)
 
-    extensions = load_extensions(opts.extensions)
+    try:
+        extensions = load_extensions(opts.extensions)
+    except ExtensionError as error:
+        print(f"griffe: error: {error}", file=sys.stderr)
+        return 1
 
-    if opts.async_loader:
-        loop = asyncio.get_event_loop()
-        coroutine = _load_packages_async(opts.packages, extensions, search, opts.docstyle, opts.docopts)
-        packages = loop.run_until_complete(coroutine)
-    else:
-        packages = _load_packages(opts.packages, extensions, search, opts.docstyle, opts.docopts)
+    packages = _load_packages(opts.packages, extensions, search, opts.docstyle, opts.docopts)
 
     if per_package_output:
         for package_name, data in packages.items():
