@@ -17,7 +17,7 @@ from typing import Any, Callable, cast
 from griffe.collections import LinesCollection, ModulesCollection
 from griffe.docstrings.dataclasses import DocstringSection
 from griffe.docstrings.parsers import Parser, parse  # noqa: WPS347
-from griffe.exceptions import AliasResolutionError, BuiltinModuleError, NameResolutionError
+from griffe.exceptions import AliasResolutionError, BuiltinModuleError, CyclicAliasError, NameResolutionError
 from griffe.expressions import Expression, Name
 from griffe.mixins import GetMembersMixin, ObjectAliasMixin, SetMembersMixin
 
@@ -339,8 +339,15 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin):
     def __repr__(self) -> str:
         return f"<{self.__class__.__name__}({self.name!r}, {self.lineno!r}, {self.endlineno!r})>"
 
-    def __bool__(self):
-        return bool(self.docstring) or any(self.members.values())
+    @property
+    def has_docstring(self) -> bool:
+        """Tell if this object has a non-empty docstring."""
+        return bool(self.docstring)  # noqa: DAR201
+
+    @property
+    def has_docstrings(self) -> bool:
+        """Tell if this object or any of its members has a non-empty docstring."""
+        return self.has_docstring or any(member.has_docstrings for member in self.members.values())  # noqa: DAR201
 
     def member_is_exported(self, member: Object | Alias, explicitely: bool = True) -> bool:
         """Tell if a member of this object is "exported".
@@ -460,7 +467,6 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin):
             return self.parent.module
         raise ValueError
 
-    # TODO: rename to top_module (for packages collection and package property)
     @cached_property
     def package(self) -> Module:
         """Return the absolute top module (the package) of this object.
@@ -516,7 +522,7 @@ class Object(GetMembersMixin, SetMembersMixin, ObjectAliasMixin):
         Returns:
             A dotted path.
         """
-        if not self.parent:
+        if self.parent is None:
             return self.name
         return ".".join((self.parent.path, self.name))
 
@@ -704,15 +710,26 @@ class Alias(ObjectAliasMixin):
         else:
             self._target = target
             self._target_path = target.path
-            if self.parent is not None:
-                target.aliases[self.path] = self
+            if parent is not None:
+                with suppress(AliasResolutionError):
+                    target.aliases[self.path] = self
         self.lineno: int | None = lineno
         self.endlineno: int | None = endlineno
         self._parent: Module | Class | None = parent
+        self._passed_through: bool = False
 
     def __getattr__(self, name: str) -> Any:
         # forward everything to the target
-        return getattr(self.target, name)
+        if self._passed_through:
+            raise CyclicAliasError([self._target_path])
+        self._passed_through = True
+        try:
+            attr = getattr(self.target, name)
+        except CyclicAliasError as error:
+            raise CyclicAliasError([self._target_path] + error.chain)
+        finally:
+            self._passed_through = False
+        return attr
 
     def __getitem__(self, key):
         # not handled by __getattr__
@@ -748,7 +765,8 @@ class Alias(ObjectAliasMixin):
     def parent(self, value: Module | Class) -> None:
         self._parent = value
         if self.resolved:
-            self._target.aliases[self.path] = self  # type: ignore[union-attr]  # we just checked the target is not None
+            with suppress(AliasResolutionError):
+                self._target.aliases[self.path] = self  # type: ignore[union-attr]  # we just checked the target is not None
 
     @cached_property
     def path(self) -> str:
@@ -815,6 +833,17 @@ class Alias(ObjectAliasMixin):
         """
         return self._target is not None
 
+    @cached_property
+    def wildcard(self) -> str | None:
+        """Return the module on which the wildcard import is performed (if any).
+
+        Returns:
+            The wildcard imported module, or None.
+        """
+        if self.name.endswith("/*"):
+            return self._target_path
+        return None
+
     def as_dict(self, full: bool = False, **kwargs: Any) -> dict[str, Any]:
         """Return this alias' data as a dictionary.
 
@@ -916,7 +945,7 @@ class Module(Object):
             True or False.
         """
         try:
-            return not self.parent and self.filepath.is_dir()
+            return self.parent is None and self.filepath.is_dir()
         except BuiltinModuleError:
             return False
 
