@@ -12,10 +12,11 @@ fastapi = griffe.load_module("fastapi")
 
 from __future__ import annotations
 
-import asyncio
+import os
 import sys
 import traceback
 from contextlib import suppress
+from functools import lru_cache
 from pathlib import Path
 from typing import Any, Iterator, Sequence, Tuple
 
@@ -23,9 +24,9 @@ from griffe.agents.extensions import Extensions
 from griffe.agents.inspector import inspect
 from griffe.agents.visitor import patch_ast, visit
 from griffe.collections import LinesCollection, ModulesCollection
-from griffe.dataclasses import Module, Object
+from griffe.dataclasses import Alias, Kind, Module, Object
 from griffe.docstrings.parsers import Parser
-from griffe.exceptions import AliasResolutionError, UnhandledPthFileError, UnimportableModuleError
+from griffe.exceptions import AliasResolutionError, CyclicAliasError, UnhandledPthFileError, UnimportableModuleError
 from griffe.logger import get_logger
 
 NamePartsType = Tuple[str, ...]
@@ -33,29 +34,43 @@ NamePartsAndPathType = Tuple[NamePartsType, Path]
 
 logger = get_logger(__name__)
 
+_accepted_py_module_extensions = [".py", ".pyc", ".pyo", ".pyd", ".so"]
+_extensions_set = set(_accepted_py_module_extensions)
 
-async def _read_async(path):
-    async with aopen(path) as fd:
-        return await fd.read()
+# TODO: namespace packages can span multiple locations! we must support it.
+# ideally: find all locations, sort them, then reverse-merge their file lists
+# (sure about sorting? yes: https://github.com/python/cpython/blob/3.10/Lib/pkgutil.py#L155,
+# and we could say "but it's locale-dependent!", but it's not an issue since our process
+# will use the same locale anyway, so the behavior will be as expected)
+# when iterating on multiple locations, if one has an __init__ module,
+# just return this one, as it takes precedence as a regular package
 
 
-async def _read_sync(path):
-    return path.read_text()
+@lru_cache(maxsize=1)
+def _get_async_reader():
+    try:  # noqa: WPS503 (false-positive)
+        from aiofiles import open as aopen
+    except ModuleNotFoundError:
+        logger.warning("aiofiles is not installed, fallback to blocking read")
 
+        async def _read_async(path):  # noqa: WPS430
+            return path.read_text()
 
-try:
-    from aiofiles import open as aopen
-except ModuleNotFoundError:
-    logger.warning("aiofiles is not installed, fallback to blocking read")
-    read_async = _read_sync
-else:
-    read_async = _read_async
+    else:
+
+        async def _read_async(path):  # noqa: WPS430,WPS440
+            async with aopen(path) as fd:
+                return await fd.read()
+
+    return _read_async
 
 
 _builtin_modules: set[str] = set(sys.builtin_module_names)
 
 
-class _BaseGriffeLoader:
+class GriffeLoader:
+    """The Griffe loader, allowing to load data from modules."""
+
     def __init__(
         self,
         extensions: Extensions | None = None,
@@ -80,6 +95,205 @@ class _BaseGriffeLoader:
         self.modules_collection: ModulesCollection = modules_collection or ModulesCollection()
         patch_ast()
 
+    def load_module(
+        self,
+        module: str | Path,
+        submodules: bool = True,
+        search_paths: Sequence[str | Path] | None = None,
+        try_relative_path: bool = True,
+    ) -> Module:
+        """Load a module.
+
+        Parameters:
+            module: The module name or path.
+            submodules: Whether to recurse on the submodules.
+            search_paths: The paths to search into.
+            try_relative_path: Whether to try finding the module as a relative path.
+
+        Returns:
+            A module.
+        """
+        if module in _builtin_modules:
+            logger.debug(f"{module} is a builtin module: inspecting")
+            module_name = module
+            top_module = self._inspect_module(module)  # type: ignore[arg-type]
+        else:
+            try:
+                module_name, top_module_name, top_module_path = _top_name_and_path(
+                    module, search_paths, try_relative_path
+                )
+            except ModuleNotFoundError:
+                logger.debug(f"Could not find {module}: trying inspection")
+                module_name = module
+                top_module = self._inspect_module(module)  # type: ignore[arg-type]
+            else:
+                logger.debug(f"Found {module}: visiting")
+                top_module = self._load_module_path(top_module_name, top_module_path, submodules=submodules)
+        self.modules_collection[top_module.path] = top_module
+        return self.modules_collection[module_name]  # type: ignore[index]
+
+    def resolve_aliases(  # noqa: WPS231
+        self,
+        only_exported: bool = True,
+        only_known_modules: bool = True,
+        max_iterations: int | None = None,
+    ) -> tuple[set[str], int]:
+        """Resolve aliases.
+
+        Parameters:
+            only_exported: When true, only try to resolve an alias if it is explicitely exported.
+            only_known_modules: When true, don't try to load unspecified modules to resolve aliases.
+            max_iterations: Maximum number of iterations on the loader modules collection.
+
+        Returns:
+            The unresolved aliases and the number of iterations done.
+        """
+        if max_iterations is None:
+            max_iterations = float("inf")  # type: ignore[assignment]
+        prev_unresolved: set[str] = set()
+        unresolved: set[str] = set("0")  # init to enter loop
+        iterations = 0
+        collection = self.modules_collection.members
+        while unresolved and unresolved != prev_unresolved and iterations < max_iterations:  # type: ignore[operator]
+            prev_unresolved = unresolved
+            unresolved = set()
+            iterations += 1
+            for module_name in list(collection.keys()):
+                module = collection[module_name]
+                unresolved |= self.resolve_module_aliases(module, only_exported, only_known_modules)
+            logger.debug(f"Iteration {iterations}")
+            logger.debug(f"prev: {len(prev_unresolved)}; current: {len(unresolved)}")
+            logger.debug("\n- " + "\n- ".join(sorted(unresolved & prev_unresolved)))
+        return unresolved, iterations
+
+    def resolve_module_aliases(  # noqa: WPS231
+        self,
+        obj: Object,
+        only_exported: bool = True,
+        only_known_modules: bool = True,
+        seen: set | None = None,
+    ) -> set[str]:
+        """Follow aliases: try to recursively resolve all found aliases.
+
+        Parameters:
+            obj: The object and its members to recurse on.
+            only_exported: When true, only try to resolve an alias if it is explicitely exported.
+            only_known_modules: When true, don't try to load unspecified modules to resolve aliases.
+            seen: Used to avoid infinite recursion.
+
+        Returns:
+            True if everything was resolved, False otherwise.
+        """
+        unresolved = set()
+        expanded = {}
+        to_remove = []
+        seen = seen or set()
+        seen.add(obj.path)
+
+        # iterate a first time to expand wildcards
+        for member in obj.members.values():
+            if member.is_alias and member.wildcard:  # type: ignore[union-attr]  # we know it's an alias
+                package = member.wildcard.split(".", 1)[0]  # type: ignore[union-attr]
+                if obj.package.path != package and package not in self.modules_collection:
+                    try:
+                        self.load_module(package, try_relative_path=False)
+                    except ImportError as error:
+                        logger.debug(f"Could not expand wildcard import {member.name} in {obj.path}: {error}")
+                    else:
+                        expanded.update(self._expand_wildcard(member))  # type: ignore[arg-type]
+                        to_remove.append(member.name)
+
+        for name in to_remove:
+            del obj[name]  # noqa: WPS420
+        for new_member in expanded.values():
+            if new_member.is_alias and not new_member.wildcard:  # type: ignore[union-attr]
+                try:
+                    alias = Alias(new_member.name, new_member.target)  # type: ignore[union-attr]
+                except AliasResolutionError:
+                    alias = Alias(new_member.name, new_member._target_path)  # type: ignore[union-attr]  # noqa: WPS437
+                except CyclicAliasError as error:  # noqa: WPS440
+                    logger.debug(str(error))
+            else:
+                alias = Alias(new_member.name, new_member)
+            obj[new_member.name] = alias
+
+        # iterate a second time to resolve aliases and recurse
+        for member in obj.members.values():  # noqa: WPS440
+            if member.is_alias:
+                if member.wildcard or member.resolved:  # type: ignore[union-attr]
+                    continue
+                if only_exported and not member.is_explicitely_exported:
+                    continue
+                try:
+                    member.resolve_target()  # type: ignore[union-attr]
+                except AliasResolutionError as error:  # noqa: WPS440
+                    path = member.path
+                    target = error.target_path  # type: ignore[union-attr]  # noqa: WPS437
+                    logger.debug(f"Alias resolution error for {path} -> {target}")
+                    unresolved.add(path)
+                    package = target.split(".", 1)[0]
+                    load_module = (
+                        not only_known_modules
+                        and obj.package.path != package
+                        and package not in self.modules_collection
+                    )
+                    if load_module:
+                        try:  # noqa: WPS505
+                            self.load_module(package, try_relative_path=False)
+                        except ImportError as error:  # noqa: WPS440
+                            logger.debug(f"Could not follow alias {member.path}: {error}")
+                except CyclicAliasError as error:
+                    logger.debug(str(error))
+                else:
+                    logger.debug(f"Alias {member.path} was resolved to {member.target.path}")  # type: ignore[union-attr]
+            elif member.kind in {Kind.MODULE, Kind.CLASS} and member.path not in seen:
+                unresolved |= self.resolve_module_aliases(member, only_exported, only_known_modules, seen)  # type: ignore[arg-type]
+
+        return unresolved
+
+    def _load_module_path(
+        self,
+        module_name: str,
+        module_path: Path,
+        submodules: bool = True,
+        parent: Module | None = None,
+    ) -> Module:
+        logger.debug(f"Loading path {module_path}")
+        if module_path.is_dir():
+            module = self._create_module(module_name, module_path)
+        elif module_path.suffix == ".py":
+            code = module_path.read_text()
+            module = self._visit_module(code, module_name, module_path, parent)
+        else:
+            module = self._inspect_module(module_name, module_path, parent)
+        if submodules:
+            self._load_submodules(module)
+        return module
+
+    def _load_submodules(self, module: Module) -> None:
+        for subparts, subpath in sorted(iter_submodules(module.filepath), key=_module_depth):
+            self._load_submodule(module, subparts, subpath)
+
+    def _load_submodule(self, module: Module, subparts: NamePartsType, subpath: Path) -> None:
+        try:
+            member_parent = self._member_parent(module, subparts, subpath)
+        except UnimportableModuleError as error:
+            logger.debug(f"{error}. Missing __init__ module?")
+            return
+        try:  # noqa: WPS225
+            member_parent[subparts[-1]] = self._load_module_path(
+                subparts[-1], subpath, submodules=False, parent=member_parent
+            )
+        except SyntaxError:
+            message = traceback.format_exc(limit=0).replace("SyntaxError: invalid syntax", "").strip()
+            logger.error(f"Syntax error: {message}")
+        except ImportError as error:  # noqa: WPS440
+            logger.error(f"Import error: {error}")
+        except UnicodeDecodeError as error:  # noqa: WPS440
+            logger.error(f"UnicodeDecodeError when loading {subpath}: {error}")
+        except OSError as error:  # noqa: WPS440
+            logger.error(f"OSError when loading {subpath}: {error}")
+
     def _create_module(self, module_name: str, module_path: Path) -> Module:
         return Module(
             module_name,
@@ -103,15 +317,18 @@ class _BaseGriffeLoader:
         )
 
     def _inspect_module(self, module_name: str, filepath: Path | None = None, parent: Module | None = None) -> Module:
-        return inspect(
-            module_name,
-            filepath=filepath,
-            extensions=self.extensions,
-            parent=parent,
-            docstring_parser=self.docstring_parser,
-            docstring_options=self.docstring_options,
-            lines_collection=self.lines_collection,
-        )
+        try:
+            return inspect(
+                module_name,
+                filepath=filepath,
+                extensions=self.extensions,
+                parent=parent,
+                docstring_parser=self.docstring_parser,
+                docstring_options=self.docstring_options,
+                lines_collection=self.lines_collection,
+            )
+        except SystemExit as error:
+            raise ImportError(f"Importing '{module_name}' raised a system exit") from error
 
     def _member_parent(self, module: Module, subparts: NamePartsType, subpath: Path) -> Module:
         parent_parts = subparts[:-1]
@@ -129,192 +346,22 @@ class _BaseGriffeLoader:
                 return member_parent
         raise UnimportableModuleError(f"{subpath} is not importable")
 
-
-class GriffeLoader(_BaseGriffeLoader):
-    """The Griffe loader, allowing to load data from modules."""
-
-    def load_module(
-        self,
-        module: str | Path,
-        submodules: bool = True,
-        search_paths: Sequence[str | Path] | None = None,
-    ) -> Module:
-        """Load a module.
-
-        Parameters:
-            module: The module name or path.
-            submodules: Whether to recurse on the submodules.
-            search_paths: The paths to search into.
-
-        Returns:
-            A module.
-        """
-        if module in _builtin_modules:
-            module_name = module
-            top_module = self._inspect_module(module)  # type: ignore[arg-type]
-        else:
-            module_name, top_module_name, top_module_path = _top_name_and_path(module, search_paths)
-            top_module = self._load_module_path(top_module_name, top_module_path, submodules=submodules)
-        self.modules_collection[top_module.path] = top_module
-        return self.modules_collection[module_name]  # type: ignore[index]
-
-    def follow_aliases(self, obj: Object, only_exported: bool = True) -> bool:  # noqa: WPS231
-        """Follow aliases: try to recursively resolve all found aliases.
-
-        Parameters:
-            obj: The object and its members to recurse on.
-            only_exported: Only try to resolve an alias if it is explicitely exported.
-
-        Returns:
-            True if everything was resolved, False otherwise.
-        """
-        success = True
-        for member in obj.members.values():
-            if member.is_alias:
-                if only_exported and not obj.member_is_exported(member, explicitely=True):
-                    continue
-                try:
-                    member.resolve_target()  # type: ignore[union-attr]  # we know it's an alias
-                except AliasResolutionError as error:
-                    success = False
-                    package = error.target_path.split(".", 1)[0]
-                    if obj.package.path != package and package not in self.modules_collection:
-                        with suppress(ModuleNotFoundError):
-                            self.load_module(package)
-            else:
-                success &= self.follow_aliases(member)  # type: ignore[arg-type]  # we know it's an object
-        return success
-
-    def _load_module_path(
-        self,
-        module_name: str,
-        module_path: Path,
-        submodules: bool = True,
-        parent: Module | None = None,
-    ) -> Module:
-        logger.debug(f"Loading path {module_path}")
-        try:
-            code = module_path.read_text()
-        except OSError:
-            module = self._create_module(module_name, module_path)
-        else:
-            module = self._visit_module(code, module_name, module_path, parent)
-        if submodules:
-            self._load_submodules(module)
-        return module
-
-    def _load_submodules(self, module: Module) -> None:
-        for subparts, subpath in sorted(iter_submodules(module.filepath), key=_module_depth):
-            self._load_submodule(module, subparts, subpath)
-
-    def _load_submodule(self, module: Module, subparts: NamePartsType, subpath: Path) -> None:
-        try:
-            member_parent = self._member_parent(module, subparts, subpath)
-        except UnimportableModuleError as error:
-            logger.warning(f"{error}. Missing __init__ module?")
-            return
-        try:
-            member_parent[subparts[-1]] = self._load_module_path(
-                subparts[-1], subpath, submodules=False, parent=member_parent
-            )
-        except SyntaxError:
-            message = traceback.format_exc(limit=0).replace("SyntaxError: invalid syntax", "").strip()
-            logger.error(f"Syntax error: {message}")
-
-
-class AsyncGriffeLoader(_BaseGriffeLoader):
-    """The asynchronous Griffe loader, allowing to load data from modules."""
-
-    async def load_module(
-        self,
-        module: str | Path,
-        submodules: bool = True,
-        search_paths: Sequence[str | Path] | None = None,
-    ) -> Module:
-        """Load a module.
-
-        Parameters:
-            module: The module name or path.
-            submodules: Whether to recurse on the submodules.
-            search_paths: The paths to search into.
-
-        Returns:
-            A module.
-        """
-        module_name, top_module_name, top_module_path = _top_name_and_path(module, search_paths)
-        top_module = await self._load_module_path(top_module_name, top_module_path, submodules=submodules)
-        self.modules_collection[top_module.path] = top_module
-        return self.modules_collection[module_name]
-
-    async def follow_aliases(self, obj: Object, only_exported: bool = True) -> bool:  # noqa: WPS231
-        """Follow aliases: try to recursively resolve all found aliases.
-
-        Parameters:
-            obj: The object and its members to recurse on.
-            only_exported: Only try to resolve an alias if it is explicitely exported.
-
-        Returns:
-            True if everything was resolved, False otherwise.
-        """
-        success = True
-        for member in obj.members.values():
-            if member.is_alias:
-                if only_exported and not obj.member_is_exported(member, explicitely=True):
-                    continue
-                try:
-                    member.resolve_target()  # type: ignore[union-attr]  # we know it's an alias
-                except AliasResolutionError as error:
-                    success = False
-                    package = error.target_path.split(".", 1)[0]
-                    if obj.package.path != package and package not in self.modules_collection:
-                        with suppress(ModuleNotFoundError):
-                            await self.load_module(package)
-            else:
-                success &= await self.follow_aliases(member)  # type: ignore[arg-type]  # we know it's an object
-        return success
-
-    async def _load_module_path(
-        self,
-        module_name: str,
-        module_path: Path,
-        submodules: bool = True,
-        parent: Module | None = None,
-    ) -> Module:
-        logger.debug(f"Loading path {module_path}")
-        try:
-            code = await read_async(module_path)
-        except OSError:
-            module = self._create_module(module_name, module_path)
-        else:
-            module = self._visit_module(code, module_name, module_path, parent)
-        if submodules:
-            await self._load_submodules(module)
-        return module
-
-    async def _load_submodules(self, module: Module) -> None:
-        await asyncio.gather(
-            *[
-                self._load_submodule(module, subparts, subpath)
-                for subparts, subpath in sorted(iter_submodules(module.filepath), key=_module_depth)
-            ]
-        )
-
-    async def _load_submodule(self, module: Module, subparts: NamePartsType, subpath: Path) -> None:
-        try:
-            member_parent = self._member_parent(module, subparts, subpath)
-        except UnimportableModuleError as error:
-            logger.debug(str(error))
-        else:
-            member_parent[subparts[-1]] = await self._load_module_path(
-                subparts[-1], subpath, submodules=False, parent=member_parent
-            )
+    def _expand_wildcard(self, wildcard_obj: Alias) -> dict[str, Object | Alias]:
+        module = self.modules_collection[wildcard_obj.wildcard]  # type: ignore[index]  # we know it's a wildcard
+        explicitely = "__all__" in module.members
+        return {
+            name: imported_member
+            for name, imported_member in module.members.items()
+            if imported_member.is_exported(explicitely=explicitely)
+        }
 
 
 def _top_name_and_path(
     module: str | Path,
     search_paths: Sequence[str | Path] | None = None,
+    try_relative_path: bool = True,
 ) -> tuple[str, str, Path]:
-    module_name, module_path = find_module_or_path(module, search_paths)
+    module_name, module_path = find_module_or_path(module, search_paths, try_relative_path)
     module_parts = module_name.split(".")
     top_module_name = module_parts[0]
     top_module_path = module_path
@@ -326,6 +373,7 @@ def _top_name_and_path(
 def find_module_or_path(
     module: str | Path,
     search_paths: Sequence[str | Path] | None = None,
+    try_relative_path: bool = True,
 ) -> tuple[str, Path]:
     """Find the name and path of a module.
 
@@ -336,6 +384,8 @@ def find_module_or_path(
     Parameters:
         module: The module name or path.
         search_paths: The paths to search into.
+        try_relative_path: Whether to try finding the module as a relative path,
+            when the given module is not already a path.
 
     Raises:
         FileNotFoundError: When a Path was passed and the module could not be found:
@@ -357,21 +407,25 @@ def find_module_or_path(
     if isinstance(module, Path):
         # programatically passed a Path, try only that
         module_name, module_path = _module_name_path(module)
-    else:
+    elif try_relative_path:
         # passed a string (from CLI or Python code), try both
         try:
             module_name, module_path = _module_name_path(Path(module))
         except FileNotFoundError:
             module_name = module
             module_path = find_module(module_name, search_paths=search_paths)
+    else:
+        module_name = module
+        module_path = find_module(module_name, search_paths=search_paths)
     return module_name, module_path
 
 
-def _module_name_path(path: Path) -> tuple[str, Path]:
+def _module_name_path(path: Path) -> tuple[str, Path]:  # noqa: WPS231
     if path.is_dir():
-        module_path = path / "__init__.py"
-        if module_path.exists():
-            return path.name, module_path
+        for ext in _accepted_py_module_extensions:
+            module_path = path / f"__init__{ext}"
+            if module_path.exists():
+                return path.name, module_path
         raise FileNotFoundError
     if path.exists():
         if path.stem == "__init__":
@@ -408,7 +462,7 @@ def find_module(module_name: str, search_paths: Sequence[str | Path] | None = No
         if abs_top_pth.exists():
             with suppress(UnhandledPthFileError):
                 location = _handle_pth_file(abs_top_pth)
-                if location.suffix == ".py":
+                if location.suffix:
                     location = location.parent
                 search = [location.parent]
                 # TODO: possible optimization
@@ -417,6 +471,7 @@ def find_module(module_name: str, search_paths: Sequence[str | Path] | None = No
 
     # resume regular search
     filepaths = [
+        # TODO: handle .py[cod] and .so files?
         Path(*parts, "__init__.py"),
         Path(*parts[:-1], f"{parts[-1]}.py"),
         Path(*parts[:-1], f"{parts[-1]}.pth"),
@@ -444,6 +499,7 @@ def _handle_pth_file(path):
     instructions = path.read_text().strip("\n").split(";")
 
     filepaths = [
+        # TODO: handle .py[cod] and .so files?
         Path(instructions[0], path.stem, "__init__.py"),
         Path(instructions[0], path.stem),  # namespace packages, try last
     ]
@@ -465,36 +521,52 @@ def _handle_pth_file(path):
     raise UnhandledPthFileError(path)
 
 
-def iter_submodules(path: Path) -> Iterator[NamePartsAndPathType]:  # noqa: WPS234
+def _filter_py_modules(path: Path) -> Iterator[Path]:
+    for root, dirs, files in os.walk(path, topdown=True):
+        # optimization: modify dirs in-place to exclude __pycache__ directories
+        dirs[:] = [dir for dir in dirs if dir != "__pycache__"]  # noqa: WPS362
+        for relfile in files:
+            if os.path.splitext(relfile)[1] in _extensions_set:
+                yield Path(root, relfile)
+
+
+def iter_submodules(path: Path) -> Iterator[NamePartsAndPathType]:  # noqa: WPS231,WPS234
     """Iterate on a module's submodules, if any.
 
     Parameters:
         path: The module path.
 
     Yields:
-        name_parts: The parts of a submodule name.
-        filepath: A submodule filepath.
+        name_parts (tuple[str, ...]): The parts of a submodule name.
+        filepath (Path): A submodule filepath.
     """
-    if path.name == "__init__.py":
+    if path.stem == "__init__":
         path = path.parent
     # optimization: just check if the file name ends with .py
     # (to distinguish it from a directory),
     # not if it's an actual file
-    elif path.suffix == ".py":
+    elif path.suffix in _extensions_set:
         return
 
-    for subpath in path.rglob("*.py"):
+    for subpath in _filter_py_modules(path):
         rel_subpath = subpath.relative_to(path)
-        if rel_subpath.name == "__init__.py":
+        py_file = rel_subpath.suffix == ".py"
+        stem = rel_subpath.stem
+        if not py_file:
+            # .py[cod] and .so files look like `name.cpython-38-x86_64-linux-gnu.ext`
+            stem = stem.split(".", 1)[0]
+        if stem == "__init__":
             # optimization: since it's a relative path,
-            # if it has only one part and is named __init__.py,
+            # if it has only one part and is named __init__,
             # it means it's the starting path
             # (no need to compare it against starting path)
             if len(rel_subpath.parts) == 1:
                 continue
             yield rel_subpath.parts[:-1], subpath
-        else:
+        elif py_file:
             yield rel_subpath.with_suffix("").parts, subpath
+        else:
+            yield rel_subpath.with_name(stem).parts, subpath
 
 
 def _module_depth(name_parts_and_path: NamePartsAndPathType) -> int:
